ItemEvents.tooltip(event => {
    
    event.addAdvanced('kubejs:origin_token', (item, advanced, text) => {
        let origin = item.nbt?.Origin;
        if (origin) {
            text.add(1, [Text.of(`Consume to rapidly evolve..`)]);
            text.add(2, [Text.of('')]);
            text.add(3, [Text.of(`Contains the: `), Text.lightPurple(`${origin.split(':')[1].toUpperCase()}`).bold(), Text.of(` origin.`)])
        }
        else {
            text.add(1, [Text.of(`Consume to randomly rapidly evolve..`)]);
        }
      })

    event.addAdvanced('kubejs:tracker', (item, advanced, text) => {
        const trackingNbt = item.nbt?.tracking_data;
        if (trackingNbt) {
            const targetName = trackingNbt.targetName;
            const dimension = trackingNbt.dimension;
            const x = trackingNbt.x.toFixed(2);
            const y = trackingNbt.y.toFixed(2);
            const z = trackingNbt.z.toFixed(2);
            const lastUpdated = trackingNbt.lastUpdated;
            text.add(2, [Text.white('Dimension: ').append(Text.lightPurple(dimension.split(':')[1]))]);
            text.add(3, [Text.white('Coordinates: ').append(Text.lightPurple(`${x}, ${y}, ${z}`))]);

            const timeSinceUpdate = (Date.now() - lastUpdated) / 1000;
            if(timeSinceUpdate !== undefined && timeSinceUpdate > 5) {
                text.add(4, [Text.of('§7Last updated: ').append(Text.of(`${timeSinceUpdate.toFixed(1)} seconds ago`))]);
            }
            else {
                text.add(4, [Text.of('§7Currently tracking..')]);
            }

        } else {
            text.add(1, [Text.of('Right-click an entity to start tracking.')]);
        }
    });

    event.addAdvanced('kubejs:heart', (item, advanced, text) => {
        const owner = item.nbt?.owner;
        const name = item.nbt?.name;
        if (owner || name) {
            text.add(2, [Text.gray(`Owner: `), Text.lightPurple(`${name ? name : owner}`).bold()]);
        }
    });

    const RARITIES = [
        { value: 'apotheosis:common', chance: 0.40, color: 'gray' },
        { value: 'apotheosis:uncommon', chance: 0.30, color: 'green' },
        { value: 'apotheosis:rare', chance: 0.15, color: 'blue' },
        { value: 'apotheosis:epic', chance: 0.10, color: 'darkPurple' },
        { value: 'apotheosis:mythic', chance: 0.04, color: 'gold' },
        { value: 'apotheosis:ancient', chance: 0.01, color: 'darkRed' }
      ];
      
      const getRarityInfo = (rarity) => {
        return RARITIES.find(r => r.value === rarity) || RARITIES[0];
      };
      
      event.addAdvanced('kubejs:item_enhancer', (item, advanced, text) => {
        const rarity = item.nbt?.rarity || 'apotheosis:common';
        const rarityInfo = getRarityInfo(rarity);
        const rarityName = rarity.split(':')[1].charAt(0).toUpperCase() + rarity.split(':')[1].slice(1);
      
        // Change item name and color based on rarity\
        item.setHoverName(Text[rarityInfo.color](`${rarityName} Item Enhancer`).bold());
      
        text.add(1, [Text.gray(`Used to reroll modifiers onto items`)]);
        
        // Add rarity text with appropriate color
        text.add(2, [
          Text.gray(`Rarity: `),
          Text[rarityInfo.color](rarityName).bold(),
          Text.gray(' modifiers will be applied.')
        ]);
      
        text.add(3, []);
        text.add(4, [Text.darkGray(`[ Right-click while in off-hand to upgrade item ]`)]);
      });

      
      event.addAdvanced('kubejs:undiscovered_gem', (item, advanced, text) => {
        const rarity = item.nbt?.rarity;
        let rarityInfo, rarityName;
      
        if (rarity) {
          rarityInfo = getRarityInfo(rarity);
          rarityName = rarity.split(':')[1].charAt(0).toUpperCase() + rarity.split(':')[1].slice(1);
        } else {
          rarityInfo = { color: 'yellow' };
          rarityName = 'Random';
        }
      
        // Change item name and color based on rarity
        item.setHoverName(Text[rarityInfo.color](`Undiscovered ${rarityName} Gem`).bold());
        
        text.add(1, [Text.white(`Contains a random gem from ${rarity ? 'the equivalent' : 'a random'} tier!`)]);
        
        // Add rarity text with appropriate color
        text.add(2, [
          Text.gray(`Rarity: `),
          Text[rarityInfo.color](rarityName).bold(),
          Text.gray(rarity ? ' gems will appear.' : ' (unset, will be determined upon opening)')
        ]);
        
        text.add(3, []);
        text.add(4, [Text.darkGray(`[ Right-click item to uncover ]`)]);
      });
      
})
