/*
    CONFIGURATION
*/

const PREPARATION_TIME = 60;

const GATEWAY_SPAWN_CONFIG = {
    'F': { interval: 15 },
    'E': { interval: 30 },
    'D': { interval: 45 },
    'C': { interval: 60 },
    'B': { interval: 90 },
    'A': { interval: 120 },
    'S': { interval: 150 },
    'SS': { interval: 180 },
    'SSS': { interval: 210 },
};

/*
    CONFIGURATION
*/

// Function to get the next scheduled time for a given interval
function getNextScheduledTime(intervalMinutes) {
    const now = new Date();
    const midnight = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0, 0);
    const minutesSinceMidnight = (now - midnight) / 60000;
    const nextIntervalIndex = Math.ceil(minutesSinceMidnight / intervalMinutes);
    const nextScheduledMinutes = nextIntervalIndex * intervalMinutes;
    const nextTime = new Date(midnight.getTime() + nextScheduledMinutes * 60000);
    
    if (nextTime <= now) {
        nextTime.setDate(nextTime.getDate() + 1);
    }
    
    return nextTime;
}

// Function to schedule the next gateway for a given tier
function scheduleNextGateway(tier) {
    const config = GATEWAY_SPAWN_CONFIG[tier];
    const nextTime = getNextScheduledTime(config.interval);
    const now = new Date();
    const delayMs = nextTime.getTime() - now.getTime();
    
    console.log(`Next ${tier} tier gateway scheduled for: ${nextTime.toLocaleString()}`);
    
    Utils.server.scheduleInTicks(Math.floor(delayMs / 50), () => {
        const onlinePlayers = Utils.server.getPlayers().size();
        if (onlinePlayers > 0) {
            Utils.server.runCommandSilent(`schedule-gateway ${tier}.RANDOM_GATEWAY ${PREPARATION_TIME}`);
            console.log(`${tier} tier gateway spawning now!`);
        } else {
            console.log(`${tier} tier gateway spawn skipped due to no online players.`);
        }
        scheduleNextGateway(tier);
    });
}

ServerEvents.loaded(event => {
    console.log("Scheduling initial gateway spawns:");
    Object.keys(GATEWAY_SPAWN_CONFIG).forEach(tier => {
        scheduleNextGateway(tier);
    });
});
