const APOTHEOSIS_GEM_TYPES = [
    // Core types (weight: 3)
    { type: 'apotheosis:core/ballast', weight: 5 },
    { type: 'apotheosis:core/brawlers', weight: 5 },
    { type: 'apotheosis:core/breach', weight: 5 },
    { type: 'apotheosis:core/combatant', weight: 5 },
    { type: 'apotheosis:core/guardian', weight: 5 },
    { type: 'apotheosis:core/lightning', weight: 5 },
    { type: 'apotheosis:core/lunar', weight: 5 },
    { type: 'apotheosis:core/samurai', weight: 5 },
    { type: 'apotheosis:core/slipstream', weight: 5 },
    { type: 'apotheosis:core/solar', weight: 5 },
    { type: 'apotheosis:core/splendor', weight: 5 },
    { type: 'apotheosis:core/tyrannical', weight: 5 },
    { type: 'apotheosis:core/warlord', weight: 5 },
  
    // Overworld types (weight: 3)
    { type: 'apotheosis:overworld/earth', weight: 3 },
    { type: 'apotheosis:overworld/royalty', weight: 3 },
  
    // The End types (weight: 3)
    { type: 'apotheosis:the_end/endersurge', weight: 1 },
    { type: 'apotheosis:the_end/mageslayer', weight: 1 },
  
    // The Nether types (weight: 3)
    { type: 'apotheosis:the_nether/blood_lord', weight: 2 },
    { type: 'apotheosis:the_nether/inferno', weight: 2 },
  
    // Ascended additional attributes (weight: 3)
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_blood', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_eldritch', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_ender', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_evocation', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_fire', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_holy', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_ice', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_lightning', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/innate_nature', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/power_eldritch', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_blood', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_eldritch', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_ender', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_evocation', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_fire', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_holy', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_ice', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_lightning', weight: 1 },
    { type: 'apotheosis_ascended:additional_attributes_iss/spell_nature', weight: 1 },

    // Irons spellbooks (weight: 2)
    { type: 'irons_spellbooks:blood', weight: 2 },
    { type: 'irons_spellbooks:cast_time', weight: 2 },
    { type: 'irons_spellbooks:cooldown', weight: 2 },
    { type: 'irons_spellbooks:ender', weight: 2 },
    { type: 'irons_spellbooks:evocation', weight: 2 },
    { type: 'irons_spellbooks:fire', weight: 2 },
    { type: 'irons_spellbooks:holy', weight: 2 },
    { type: 'irons_spellbooks:ice', weight: 2 },
    { type: 'irons_spellbooks:intelligent', weight: 2 },
    { type: 'irons_spellbooks:lightning', weight: 2 },
    { type: 'irons_spellbooks:nature', weight: 2 },
    { type: 'irons_spellbooks:spell_resist', weight: 2 },
    { type: 'irons_spellbooks:summoning', weight: 2 }
  ];
  
const RARITIES = [
    { value: 'apotheosis:common', chance: 0.40, color: 'gray' },
    { value: 'apotheosis:uncommon', chance: 0.30, color: 'green' },
    { value: 'apotheosis:rare', chance: 0.15, color: 'blue' },
    { value: 'apotheosis:epic', chance: 0.10, color: 'dark_purple' },
    { value: 'apotheosis:mythic', chance: 0.04, color: 'gold' },
    { value: 'apotheosis:ancient', chance: 0.01, color: 'dark_red' }
];
  
// Function to select a weighted random rarity
function selectWeightedRarity() {
    const totalWeight = RARITIES.reduce((sum, rarity) => sum + rarity.chance, 0);
    let random = Math.random() * totalWeight;
    
    for (const rarity of RARITIES) {
        if (random < rarity.chance) {
            return rarity.value;
        }
        random -= rarity.chance;
    }
    
    // Fallback to common if something goes wrong
    return 'apotheosis:common';
}

// Function to select a weighted random gem type
function selectWeightedGemType() {
    const totalWeight = APOTHEOSIS_GEM_TYPES.reduce((sum, gem) => sum + gem.weight, 0);
    let random = Math.random() * totalWeight;
    
    for (const gem of APOTHEOSIS_GEM_TYPES) {
        if (random < gem.weight) {
            return gem.type;
        }
        random -= gem.weight;
    }
    
    // Fallback to the first gem type if something goes wrong
    return APOTHEOSIS_GEM_TYPES[0].type;
}

function generateRandomGem() {
    const randomGemType = selectWeightedGemType();
    const randomRarity = selectWeightedRarity();
    
    const gemItem = Item.of('apotheosis:gem');
    gemItem.nbt = {
        gem: randomGemType,
        affix_data: { rarity: randomRarity }
    };
    
    return {
        item: gemItem,
        type: randomGemType,
        rarity: randomRarity
    };
}

function shouldBold(rarity) {
    return !(rarity === 'apotheosis:common' || rarity === 'apotheosis:uncommon' || rarity === 'apotheosis:rare')
}

ItemEvents.rightClicked('kubejs:undiscovered_gem', event => {
    const player = event.player;
    let rarity = event.item.nbt?.rarity;

    // If rarity is unset, select a random rarity
    if (!rarity) {
        rarity = selectWeightedRarity();
    }

    // Generate a random gem of the selected rarity
    const { item: gemItem, type: gemType } = generateRandomGem();
    gemItem.nbt.affix_data.rarity = rarity;

    // Give the player the gem
    player.give(gemItem);

    // Remove the undiscovered gem from the player's inventory
    event.item.count--;

    // Send a formatted chat message
    const rarityInfo = RARITIES.find(r => r.value === rarity);
    const rarityName = rarity.split(':')[1].charAt(0).toUpperCase() + rarity.split(':')[1].slice(1);
    const gemTypeName = gemType.split('/').pop().replace(/_/g, ' ');
    
    const message = Text.of(`Discovered ${rarityName} ${gemTypeName} Gem!`)
        .color(rarityInfo.color)
        .bold(shouldBold(rarityInfo.value));

    player.tell(message);

    // Announce to all players if rarity is Epic or above
    if (rarityInfo.value === 'apotheosis:epic' || rarityInfo.value === 'apotheosis:mythic' || rarityInfo.value === 'apotheosis:ancient') {    
        player.getServer().players.forEach(p => {
            p.tell(
                Text.of(`${player.getName().getString()}`)
                    .color('light_purple')
                    .append(Text.white(` has discovered an `))
                    .append(Text[rarityInfo.color.replace(/_([a-z])/g, (_, letter) => letter.toUpperCase())](`${rarityName} `).bold())
                    .append(Text.white(`${gemTypeName} Gem`))
            );
        });
    }
    
});


// Handle right-clicking the item enhancer
ItemEvents.rightClicked('kubejs:item_enhancer', event => {
    let player = event.player;
    let offHandItem = player.getHeldItem('off_hand');

    if (offHandItem.id === 'kubejs:item_enhancer') {
        let rarity = offHandItem.nbt?.rarity || selectWeightedRarity();

        let result = player.runCommandSilent(`apoth lootify ${rarity}`);
        player.tell(`Applied the modification to your held-item ;)`);
        offHandItem.count--;

        // Send a formatted chat message
        let rarityInfo = RARITIES.find(r => r.value === rarity);
        let rarityName = rarity.split(':')[1].charAt(0).toUpperCase() + rarity.split(':')[1].slice(1);

        // Announce to all players if rarity is Epic or above
        if (rarityInfo.value === 'apotheosis:epic' || rarityInfo.value === 'apotheosis:mythic' || rarityInfo.value === 'apotheosis:ancient') {
            player.getServer().players.forEach(p => {
                p.tell(
                    Text.of(`${player.getName().getString()}`)
                        .color('light_purple')
                        .append(Text.white(` has applied an upgrade with rarity: `))
                        .append(Text[rarityInfo.color.replace(/_([a-z])/g, (_, letter) => letter.toUpperCase())](`${rarityName}`).bold())
                );
            });
        }
    }
    // If the item is not in the off-hand, do nothing
});