const kits = JsonIO.read('config/HaxysHideout/kits.json') || {};

const hasPermissionNode = (player, permission) => {
    const LuckPerms = Java.loadClass('net.luckperms.api.LuckPermsProvider');
    const userManager = LuckPerms.get().getUserManager();
    return userManager['getUser(java.util.UUID)'](player.uuid).getCachedData().getPermissionData().checkPermission(permission).asBoolean();
}

function canClaimKit(player, kitName, source) {
    if (source.hasPermission(2)) return true;
    const result = hasPermissionNode(player, `kit.claim.${kitName}`)
    if(!result) return false;

    const lastClaimed = player.persistentData.getLong(`lastClaimed_${kitName}`);
    const cooldown = kits[kitName].cooldownMinutes * 60 * 1000;
    return Date.now() - lastClaimed >= cooldown;
}

function giveKit(player, kitName) {
    const kit = kits[kitName];
    if (!kit) {
        console.log(`[Kits] Attempted to give non-existent kit: ${kitName}`);
        return false;
    }

    kit.Inventory.forEach(item => {
        const itemStack = Item.of(item.id, item.Count);
        if (!itemStack) {
            console.log(`[Kits] Item ${item.id} does not exist in kit: ${kitName}`);
            return;
        }

        if (item.tag) {
            itemStack.nbt = item.tag;
        }
        player.give(itemStack);
    });

    player.persistentData.putLong(`lastClaimed_${kitName}`, Date.now());
    console.log(`[Kits] Kit ${kitName} given to player ${player.name}`);
    return true;
}

function formatTimeRemaining(milliseconds) {
    const seconds = Math.floor(milliseconds / 1000);
    const minutes = Math.floor(seconds / 60);
    const hours = Math.floor(minutes / 60);
    const days = Math.floor(hours / 24);

    if (days > 0) return `${days}d ${hours % 24}h`;
    if (hours > 0) return `${hours}h ${minutes % 60}m`;
    if (minutes > 0) return `${minutes}m ${seconds % 60}s`;
    return `${seconds}s`;
}

function createKitHoverText(kitName) {
    const kit = kits[kitName];
    if (!kit) return '';

    let hoverText = `§e§l${kitName} Kit\n§7Contents:\n`;
    kit.Inventory.forEach(item => {
        hoverText += `§8• §f${item.Count}x ${item.id}\n`;
    });
    return hoverText;
}

ServerEvents.commandRegistry(event => {
    const { commands: Commands, arguments: Arguments } = event;

    event.register(
        Commands.literal('kit')
            .requires(source => source.hasPermission(2))
            .then(Commands.argument('kitName', Arguments.STRING.create(event))
                .suggests((context, builder) => {
                    Object.keys(kits).forEach(kitName => {
                        if (canClaimKit(context.source.getPlayer(), kitName, context.source)) {
                            builder.suggest(kitName);
                        }
                    });
                    return builder.buildFuture();
                })
                .executes(context => {
                    const player = context.source.getPlayer();
                    const kitName = Arguments.STRING.getResult(context, 'kitName');

                    if (canClaimKit(player, kitName, context.source)) {
                        if (giveKit(player, kitName)) {
                            player.tell(Text.of(`§a✔ §eKit Claimed: §6${kitName}`));
                        } else {
                            player.tell(Text.of(`§c✘ §eError: §7Kit §6${kitName}§7 does not exist.`));
                        }
                    } else {
                        if (hasPermissionNode(player, `kit.claim.${kitName}`)) {
                            const cooldownMinutes = kits[kitName].cooldownMinutes;
                            const lastClaimed = player.persistentData.getLong(`lastClaimed_${kitName}`);
                            const timeRemaining = (lastClaimed + cooldownMinutes * 60 * 1000) - Date.now();
                            player.tell(Text.of(`§c⏳ §eCooldown: §7Kit §6${kitName}§7 is on cooldown. §6${formatTimeRemaining(timeRemaining)}§7 remaining.`));
                        } else {
                            player.tell(Text.of(`§c✘ §eError: §7You don't have permission to claim kit §6${kitName}§7.`));
                        }
                    }

                    return 1;
                })
            )
            .then(Commands.argument('player', Arguments.PLAYER.create(event))
                .requires(source => source.hasPermission(2) || source.hasPermission(3))
                .then(Commands.argument('kitName', Arguments.STRING.create(event))
                    .suggests((context, builder) => {
                        Object.keys(kits).forEach(kitName => {
                            builder.suggest(kitName);
                        });
                        return builder;
                    })
                    .executes(context => {
                        const player = Arguments.PLAYER.getResult(context, 'player');
                        const kitName = Arguments.STRING.getResult(context, 'kitName');

                        if (giveKit(player, kitName)) {
                            context.source.tell(Text.of(`§a✔ §eKit Given: §6${kitName}§7 to §6${player.name}`));
                            player.tell(Text.of(`§a✔ §eKit Received: §6${kitName}`));
                        } else {
                            context.source.tell(Text.of(`§c✘ §eError: §7Kit §6${kitName}§7 does not exist.`));
                        }

                        return 1;
                    })
                )
            )
.executes(context => {
    const player = context.source.getPlayer();
    const allKits = Object.keys(kits);
    const ownedKits = allKits.filter(kitName => hasPermissionNode(player, `kit.claim.${kitName}`));
    const otherKits = allKits.filter(kitName => !hasPermissionNode(player, `kit.claim.${kitName}`));

    let message = Text.of('');
    // Other Kits
    if (otherKits.length > 0) {
        message = message.append(Text.of('§e§l┌─ Other Kits ─┐\n'));
        otherKits.forEach(kitName => {
            message = message.append(Text.of(`  §7✘ §8${kitName}\n`)
                .hover(createKitHoverText(kitName)));
        });
        
    }

    // Your Kits
    const claimedKits = ownedKits.filter(kitName => !canClaimKit(player, kitName, context.source));
    const availableKits = ownedKits.filter(kitName => canClaimKit(player, kitName, context.source));

    if (claimedKits.length > 0 || availableKits.length > 0) {
        if (otherKits.length > 0) {
            // If Other Kits exist, add a newline for separation
            message = message.append(Text.of('\n'));
        }
        message = message.append(Text.of('§e§l┌─ Your Kits ──┐\n'));

        // Claimed kits (on cooldown)
        claimedKits.forEach(kitName => {
            const lastClaimed = player.persistentData.getLong(`lastClaimed_${kitName}`);
            const cooldown = kits[kitName].cooldownMinutes * 60 * 1000;
            const timeRemaining = (lastClaimed + cooldown) - Date.now();
            message = message.append(Text.of(`  §7• §6${kitName} §7(§e${formatTimeRemaining(timeRemaining)}§7)\n`)
                .hover(createKitHoverText(kitName)));
        });

        // Available kits
        availableKits.forEach(kitName => {
            message = message.append(Text.of(`  §7✔ §a${kitName}\n`)
                .hover(createKitHoverText(kitName)));
        });
    }

    player.tell(message);
    console.log(`[Kits] Kit list displayed for player ${player.name}`);
    return 1;
})
    );
});