// Initialize the lifesteal log array
let lifestealLog = [];

// Load existing log if it exists
const prevLog = JsonIO.read("kubejs/config/lifesteal-log.json");
if (prevLog?.length > 0) {
    lifestealLog = prevLog;
}

function logLifestealEvent(player, eventType, message) {
    const event = {
        date: new Date().toISOString(),
        eventType: eventType,
        message: message,
        player: player.getName().getString()
    };
    
    lifestealLog.push(event);
    
    // Write the updated log to the file
    JsonIO.write("kubejs/data/lifesteal.log", lifestealLog);
    
    // Log the event to the console
    console.log(`Lifesteal Event: ${eventType} - ${message} - ${player.getName().getString()}`);
}

function nonLinearMapping(x) {
  if (x <= 40) {
      return x;
  } else {
      // Parameters to adjust the curve
      const a = 150;
      const b = 0.0015;
      const c = 0.2;
      
      // Calculate the output using a modified logarithmic function
      const result = a * Math.log(1 + b * (x - 40)) + c * (x - 40) + 40;
      
      return Math.ceil(result);
  }
}

function inverseNonLinearMapping(y) {
  if (y <= 40) {
      return y;
  } else {
      // Parameters to adjust the curve (same as in nonLinearMapping)
      const a = 80;
      const b = 0.0015;
      const c = 0.2;
      
      // Inverse function of the modified logarithmic function
      const result = (Math.exp((y - 40) / a) - 1) / b + 40 - (y - 40) / c;
      
      return Math.floor(result);
  }
}

const MIN_HEALTH = 5;

function calculateHeartLoss(currentUnmappedHealth) {
  // Calculate 10% of current health, rounded down to nearest even number (full heart)
  const tenPercent = Math.floor(currentUnmappedHealth * 0.1);
  const roundedDown = Math.floor(tenPercent / 2) * 2;
  
  // Ensure we're stealing at least 1 full heart (2 HP) and not leaving the player below MIN_HEALTH
  return Math.max(2, Math.min(roundedDown, currentUnmappedHealth - MIN_HEALTH));
}

const HEART = '❤';
const WHITE = '§f';
const GRAY = '§7';
const BOLD = '§l';

function getHealthColor(health) {
    if (health <= 10) return '§c'; // Red
    if (health <= 20) return '§e'; // Yellow
    if (health <= 30) return '§a'; // Green
    if (health <= 40) return '§2'; // Dark Green
    if (health <= 50) return '§b'; // Aqua
    if (health <= 60) return '§3'; // Dark Aqua
    if (health <= 70) return '§9'; // Blue
    if (health <= 80) return '§1'; // Dark Blue
    if (health <= 90) return '§d'; // Light Purple
    return '§5'; // Dark Purple
}

function formatHealth(health, isMaxHealth) {
    if (isMaxHealth) {
        return `${BOLD}${getHealthColor(health)}${HEART}${health}§r`;
    } else {
        return `${BOLD}${WHITE}${HEART}${health}§r`;
    }
}

function formatPlayerName(name) {
    return `${BOLD}§e${name}§r`;
}

function processKill(victim, aggressor) {
  const victimUnmappedHealth = victim.getPersistentData().getInt('lifestealHealth');
  const heartLoss = calculateHeartLoss(victimUnmappedHealth);
  
  if (heartLoss >= 2) {
    const heartCount = heartLoss / 2;
    for (let i = 0; i < heartCount; i++) {
      const heartItem = Item.of('kubejs:heart');
      heartItem.nbt = { owner: victim.uuid.toString(), name: victim.getName().getString() };
      aggressor.giveInHand(heartItem);
    }

    victim.tell(Text.of(`${WHITE}You were killed by ${formatPlayerName(aggressor.getName().getString())} and lost ${formatHealth(heartCount)}!`));
    aggressor.tell(Text.of(`${WHITE}You killed ${formatPlayerName(victim.getName().getString())} and stole ${formatHealth(heartCount)}!`));

    UpdateHealth(victim, -heartLoss);
    logLifestealEvent(aggressor, 'PLAYER_KILLED', `${aggressor.getName().getString()} killed ${victim.getName().getString()} and stole ${heartCount} heart(s)`);
    logLifestealEvent(victim, 'PLAYER_DIED', `${victim.getName().getString()} died due to ${aggressor.getName().getString()} and lost ${heartCount} heart(s)`);
  } else {
    victim.tell(Text.of(`${WHITE}You were killed by ${formatPlayerName(aggressor.getName().getString())} but didn't lose any hearts.`));
    aggressor.tell(Text.of(`${WHITE}You killed ${formatPlayerName(victim.getName().getString())} but couldn't steal any hearts.`));
    logLifestealEvent(aggressor, 'PLAYER_KILLED_NO_HEARTS', `${aggressor.getName().getString()} killed ${victim.getName().getString()} but couldn't steal any hearts`);
    logLifestealEvent(victim, 'PLAYER_DIED_NO_HEARTS', `${victim.getName().getString()} died due to ${aggressor.getName().getString()} but didn't lose any hearts`);
  }
}

ServerEvents.commandRegistry(event => {
  const { commands: Commands, arguments: Arguments } = event;

  event.register(
    Commands.literal('get-health')
      .requires(s => s.hasPermission(2))
      .then(Commands.argument('target', Arguments.PLAYER.create(event))
        .executes(ctx => {
          const target = Arguments.PLAYER.getResult(ctx, 'target');
          const maxHealth = target.getMaxHealth();
          const currentHealth = target.getHealth();
          const unmappedHealth = target.getPersistentData().getInt('lifestealHealth');
          ctx.source.sendSuccess(Text.of(`${WHITE}${formatPlayerName(target.getName().toString().slice(8).slice(0, -1))}'s HP is: ${formatHealth(currentHealth.toFixed(1))}/${formatHealth(maxHealth.toFixed(1), true)} ${GRAY}(Unmapped: ${formatHealth(unmappedHealth)})${WHITE}`), false);
          logLifestealEvent(target, 'GET_HEALTH', `${target.getName().getString()} checked their health: ${currentHealth.toFixed(1)}/${maxHealth.toFixed(1)} (Unmapped: ${unmappedHealth})`);
          return 1;
        })
      )
  );

  event.register(
    Commands.literal('set-health')
      .requires(s => s.hasPermission(2))
      .then(Commands.argument('target', Arguments.PLAYER.create(event))
        .then(Commands.argument('amount', Arguments.FLOAT.create(event))
          .executes(ctx => {
            const target = Arguments.PLAYER.getResult(ctx, 'target');
            const amount = Arguments.FLOAT.getResult(ctx, 'amount');
            SetHealth(target, amount);
            ctx.source.sendSuccess(Text.of(`${WHITE}Set ${formatPlayerName(target.getName().toString().slice(8).slice(0, -1))}'s HP to ${formatHealth(amount)} ${GRAY}(Mapped: ${formatHealth(nonLinearMapping(amount), true)})${WHITE}`), false);
            return 1;
          })
        )
      )
  );

  event.register(
    Commands.literal('self-kill')
      .requires(s => s.hasPermission(2))
      .executes(ctx => {
        const player = ctx.source.player;
        processKill(player, player);
        return 1;
      })
  );

  event.register(
    Commands.literal('withdraw-heart')
      .requires(s => s.hasPermission(0))
      .executes(ctx => {
        const player = ctx.source.player;
        const currentUnmappedHealth = player.getPersistentData().getInt('lifestealHealth');
        if (currentUnmappedHealth > MIN_HEALTH + 1) {
          UpdateHealth(player, -2);
          const heartItem = Item.of('kubejs:heart');
          heartItem.nbt = { owner: player.uuid.toString(), name: player.name.getString() };
          player.give(heartItem);
          const newUnmappedHealth = currentUnmappedHealth - 2;
          const newMappedHealth = nonLinearMapping(newUnmappedHealth);
          ctx.source.sendSuccess(Text.of(`${WHITE}You withdrew ${formatHealth(1)}. Your new max health is ${formatHealth(newMappedHealth, true)} ${GRAY}(Unmapped: ${formatHealth(newUnmappedHealth)})${WHITE}`), false);
          logLifestealEvent(player, 'WITHDRAW_HEART', `${player.getName().getString()} withdrew a heart. New max health is ${newMappedHealth} (Unmapped: ${newUnmappedHealth})`);
          return 1;
        } else {
          ctx.source.sendSuccess(Text.of(`${WHITE}You don't have enough health to withdraw a heart.`), false);
          logLifestealEvent(player, 'WITHDRAW_HEART_FAILED', `${player.getName().getString()} attempted to withdraw a heart but didn't have enough health`);
          return 0;
        }
      })
  );
});

ItemEvents.rightClicked('kubejs:heart', event => {
  const { player } = event;
  const currentUnmappedHealth = player.getPersistentData().getInt('lifestealHealth');
  const nbt = event.item.getNbt();
  UpdateHealth(player, 2);
  const newUnmappedHealth = currentUnmappedHealth + 2;
  const newMappedHealth = nonLinearMapping(newUnmappedHealth);
  player.tell(Text.of(`${WHITE}You gained ${formatHealth(2)}. Your new max health is ${formatHealth(newMappedHealth, true)} ${GRAY}(Unmapped: ${formatHealth(newUnmappedHealth)})${WHITE}`));
  event.item.count--;
  player.playSound('minecraft:entity.experience_orb.pickup');
  if(nbt.name || nbt.owner) {
    let identifier = nbt.name ? `name: ${nbt.name}` : `uuid: ${nbt.owner}`;
    logLifestealEvent(player, 'HEART_CONSUMED', `${player.getName().getString()} consumed a heart from ${identifier}. New max health is ${newMappedHealth} (Unmapped: ${newUnmappedHealth})`);
  }
  else {
    logLifestealEvent(player, 'HEART_CONSUMED', `${player.getName().getString()} consumed a heart. New max health is ${newMappedHealth} (Unmapped: ${newUnmappedHealth})`);
  }

  event.cancel();
})

function AgressiveHealthSet(event, player) {
  const unmappedHealth = player.getPersistentData().getInt('lifestealHealth');
  const mappedHealth = nonLinearMapping(unmappedHealth);
  for (let i = 0; i < 160; i++) {
    if (mappedHealth > 0) {
      player.setMaxHealth(mappedHealth);
      player.setHealth(mappedHealth);
    } 

    event.server.scheduleInTicks(i, () => {
      if (mappedHealth > 0 && player.getMaxHealth() != mappedHealth) {
        player.setMaxHealth(mappedHealth);
        player.setHealth(mappedHealth);
      }
    });
  }
}

function UpdateHealth(player, difference) {
  let currentUnmappedHealth = player.getPersistentData().getInt('lifestealHealth') || inverseNonLinearMapping(player.getMaxHealth());
  let newUnmappedHealth = Math.max(currentUnmappedHealth + difference, MIN_HEALTH)
  
  player.getPersistentData().putInt('lifestealHealth', newUnmappedHealth);

  const newMappedHealth = nonLinearMapping(newUnmappedHealth);
  player.setMaxHealth(newMappedHealth);
  player.setHealth(Math.min(player.getHealth() + difference, newMappedHealth));

  logLifestealEvent(player, 'HEALTH_UPDATED', `${player.getName().getString()}'s health updated by ${difference}. New max health is ${newMappedHealth} (Unmapped: ${newUnmappedHealth})`);
}

function SetHealth(player, amount) {
  const unmappedAmount = Math.max(amount, MIN_HEALTH);
  player.getPersistentData().putInt('lifestealHealth', unmappedAmount);
  const mappedAmount = nonLinearMapping(unmappedAmount);
  player.setMaxHealth(mappedAmount);
  player.setHealth(mappedAmount);
  logLifestealEvent(player, 'HEALTH_SET', `${player.getName().getString()}'s health set to ${mappedAmount} (Unmapped: ${unmappedAmount})`);
}

EntityEvents.death(event => {
  const aggressor = event.source.player;
  const victim = event.entity;

  if (aggressor && victim.isPlayer()) {
    processKill(victim, aggressor);
  }
});

PlayerEvents.respawned(event => {
  const player = event.player;
  const unmappedHealth = player.getPersistentData().getInt('lifestealHealth');
  const mappedHealth = nonLinearMapping(unmappedHealth);
  
  let message = Text.of(`${WHITE}You respawned with: ${formatHealth(unmappedHealth)}`);
  
  if (unmappedHealth !== mappedHealth) {
    message = Text.join([
      message,
      Text.of('\n'),
      Text.of(`${GRAY}Your effective health is: ${formatHealth(mappedHealth, true)}`)
    ]);
  }
  
  player.tell(message);
  AgressiveHealthSet(event, player);
  logLifestealEvent(player, 'PLAYER_RESPAWNED', `${player.getName().getString()} respawned with ${unmappedHealth} unmapped health (Effective: ${mappedHealth})`);
});

PlayerEvents.decorateChat(event => {
    const player = event.player;
    const unmappedHealth = player.getPersistentData().getInt('lifestealHealth') || inverseNonLinearMapping(player.getMaxHealth());
    const mappedHealth = nonLinearMapping(unmappedHealth);
    event.setMessage(`${formatHealth(mappedHealth, true)} ${WHITE}${event.message}`);
})

// Event for player login to restore health
PlayerEvents.loggedIn(event => {
  const player = event.player;
  const persistentHealth = player.getPersistentData().getInt('lifestealHealth');
  
  if (persistentHealth <= 0) {
    const initialUnmappedHealth = inverseNonLinearMapping(player.getMaxHealth());
    player.getPersistentData().putInt('lifestealHealth', initialUnmappedHealth);
  }
  
  AgressiveHealthSet(event, player);
});

global.UpdateHealth = UpdateHealth;
global.SetHealth = SetHealth;
global.MIN_HEALTH = MIN_HEALTH;
